#include <stdlib.h>
#include <dos.h>
#include <conio.h>
#include "ns_dpmi.h"
#include "ns_task.h"
#include "ns_cards.h"
#include "ns_user.h"
#include "ns_pwm.h"

#include "m_misc.h"

#include "doomstat.h"

static int PCSpeakerPWM_Installed = 0;

static char *PCSpeakerPWM_BufferStart;
static char *PCSpeakerPWM_BufferEnd;
static char *PCSpeakerPWM_CurrentBuffer;
static int PCSpeakerPWM_BufferNum = 0;
static int PCSpeakerPWM_NumBuffers = 0;
static int PCSpeakerPWM_TotalBufferSize = 0;
static int PCSpeakerPWM_TransferLength = 0;
static int PCSpeakerPWM_CurrentLength = 0;

static char *PCSpeakerPWM_SoundPtr;
volatile int PCSpeakerPWM_SoundPlaying;

static task *PCSpeakerPWM_Timer;

void (*PCSpeakerPWM_CallBack)(void);

/*---------------------------------------------------------------------
   Function: PCSpeakerPWM_ServiceInterrupt

   Handles interrupt generated by sound card at the end of a voice
   transfer.  Calls the user supplied callback function.
---------------------------------------------------------------------*/

static void PCSpeakerPWM_ServiceInterrupt(task *Task)
{
    if (*PCSpeakerPWM_SoundPtr > -128)
    {
        short valueConv = *PCSpeakerPWM_SoundPtr + 128;
        unsigned char value = (unsigned char) valueConv;

        outp(0x43, 0xb0);
        outp(0x42, value >> 1);
        outp(0x42, 0);
        outp(0x61, inp(0x61) | 0x3);
    }
    else
    {
        outp(0x61, inp(0x61) & 0xfc);
    }

    PCSpeakerPWM_SoundPtr++;

    PCSpeakerPWM_CurrentLength--;
    if (PCSpeakerPWM_CurrentLength == 0)
    {
        // Keep track of current buffer
        PCSpeakerPWM_CurrentBuffer += PCSpeakerPWM_TransferLength;
        PCSpeakerPWM_BufferNum++;
        if (PCSpeakerPWM_BufferNum >= PCSpeakerPWM_NumBuffers)
        {
            PCSpeakerPWM_BufferNum = 0;
            PCSpeakerPWM_CurrentBuffer = PCSpeakerPWM_BufferStart;
        }

        PCSpeakerPWM_CurrentLength = PCSpeakerPWM_TransferLength;
        PCSpeakerPWM_SoundPtr = PCSpeakerPWM_CurrentBuffer;

        // Call the caller's callback function
        if (PCSpeakerPWM_CallBack != NULL)
        {
            PCSpeakerPWM_CallBack();
        }
    }
}

/*---------------------------------------------------------------------
   Function: PCSpeakerPWM_StopPlayback

   Ends the transfer of digitized sound to the Sound Source.
---------------------------------------------------------------------*/

void PCSpeakerPWM_StopPlayback(void)
{
    if (PCSpeakerPWM_SoundPlaying)
    {
        // Turn off
        outp(0x61, inp(0x61) & 0xfc);
        TS_Terminate(PCSpeakerPWM_Timer);
        PCSpeakerPWM_SoundPlaying = 0;
        PCSpeakerPWM_BufferStart = NULL;
    }
}

/*---------------------------------------------------------------------
   Function: PCSpeakerPWM_BeginBufferedPlayback

   Begins multibuffered playback of digitized sound on the Sound Source.
---------------------------------------------------------------------*/

int PCSpeakerPWM_BeginBufferedPlayback(
    char *BufferStart,
    int BufferSize,
    int NumDivisions,
    void (*CallBackFunc)(void))
{
    if (PCSpeakerPWM_SoundPlaying)
    {
        PCSpeakerPWM_StopPlayback();
    }

    PCSpeakerPWM_SetCallBack(CallBackFunc);

    PCSpeakerPWM_BufferStart = BufferStart;
    PCSpeakerPWM_CurrentBuffer = BufferStart;
    PCSpeakerPWM_SoundPtr = BufferStart;
    PCSpeakerPWM_TotalBufferSize = BufferSize;
    PCSpeakerPWM_BufferEnd = BufferStart + BufferSize;
    // VITI95: OPTIMIZE
    PCSpeakerPWM_TransferLength = BufferSize / NumDivisions;
    PCSpeakerPWM_CurrentLength = PCSpeakerPWM_TransferLength;
    PCSpeakerPWM_BufferNum = 0;
    PCSpeakerPWM_NumBuffers = NumDivisions;

    PCSpeakerPWM_SoundPlaying = 1;

    PCSpeakerPWM_Timer = TS_ScheduleTask(PCSpeakerPWM_ServiceInterrupt, PCSpeakerPWM_SampleRate, 1, NULL);
    TS_Dispatch();

    return (PCSpeakerPWM_Ok);
}

/*---------------------------------------------------------------------
   Function: PCSpeakerPWM_SetCallBack

   Specifies the user function to call at the end of a sound transfer.
---------------------------------------------------------------------*/

void PCSpeakerPWM_SetCallBack(void (*func)(void))
{
    PCSpeakerPWM_CallBack = func;
}

/*---------------------------------------------------------------------
   Function: PCSpeakerPWM_Init

   Initializes the Sound Source prepares the module to play digitized
   sounds.
---------------------------------------------------------------------*/

int PCSpeakerPWM_Init(int soundcard)
{
    int status;

    if (PCSpeakerPWM_Installed)
    {
        PCSpeakerPWM_Shutdown();
    }

    status = PCSpeakerPWM_Ok;

    PCSpeakerPWM_SoundPlaying = 0;

    PCSpeakerPWM_SetCallBack(NULL);

    PCSpeakerPWM_BufferStart = NULL;

    PCSpeakerPWM_Installed = 1;

    return (status);
}

int PCSpeakerPWM_SetMixMode(int mode)
{
    mode = MONO_8BIT;
    return (mode);
}

/*---------------------------------------------------------------------
   Function: PCSpeakerPWM_Shutdown

   Ends transfer of sound data to the Sound Source.
---------------------------------------------------------------------*/

void PCSpeakerPWM_Shutdown(void)
{
    PCSpeakerPWM_StopPlayback();

    PCSpeakerPWM_SoundPlaying = 0;

    PCSpeakerPWM_BufferStart = NULL;

    PCSpeakerPWM_SetCallBack(NULL);

    PCSpeakerPWM_Installed = 0;
}
